#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "tinyaes/tinyaes.h"

int
encrypt_test(unsigned char *key, int keylen,
	     unsigned char *pt, int ptlen,
	     unsigned char *ct,
	     unsigned char *result)
{
	aes_ctx *ctx;
	int key_bytes = keylen / 8;
	int i;

	if (keylen != 128 && keylen != 192 && keylen != 256) {
		printf("illegal key length");
		return 0;
	}

	printf("encrypt: ");
	for (i = 0; i < key_bytes; i++)
		key[i] = i;
	for (i = 0; i < key_bytes; i++)
		pt[i] = (i * 16) + i;
	for (i = 0; i < ptlen; i++)
		ct[i] = 0;

	ctx = aes_init_key(key, keylen);
	aes_ecb_encrypt(ctx, ct, pt, ptlen);
	aes_delete_key(ctx);

	for (i = 0; i < ptlen; i++) {
		if (ct[i] != result[i]) {
			printf("FAIL.\n");
			return 0;
			}
	}
	printf("PASS.\n");
	return 1;
}

int
decrypt_test(unsigned char *key, int keylen,
	     unsigned char *pt, int ptlen,
	     unsigned char *ct)
{
	aes_ctx *ctx;
	int key_bytes = keylen / 8;
	int i;

	if (keylen != 128 && keylen != 192 && keylen != 256) {
		printf("illegal key length");
		return 0;
	}

	printf("decrypt: ");
	for (i = 0; i < key_bytes; i++)
		key[i] = i;
	for (i = 0; i < ptlen; i++)
		pt[i] = 0;

	ctx = aes_init_key(key, keylen);
	aes_ecb_decrypt(ctx, pt, ct, ptlen);
	aes_delete_key(ctx);

	for (i = 0; i < ptlen; i++) {
		if (pt[i] != (i * 16) + i) {
			printf("FAIL.\n");
			return 0;
			}
	}
	printf("PASS.\n");
	return 1;
}

int
main()
{
	unsigned char key[32];
	unsigned char pt[32];
	unsigned char ct[32];
	unsigned char ct_result_128[16] = { 0x69, 0xc4, 0xe0, 0xd8, 0x6a,
					    0x7b, 0x04, 0x30, 0xd8, 0xcd,
					    0xb7, 0x80, 0x70, 0xb4, 0xc5,
					    0x5a };
	unsigned char ct_result_192[16] = { 0xdd, 0xa9, 0x7c, 0xa4, 0x86,
					    0x4c, 0xdf, 0xe0, 0x6e, 0xaf,
					    0x70, 0xa0, 0xec, 0x0d, 0x71,
					    0x91 };
	unsigned char ct_result_256[16] = { 0x8e, 0xa2, 0xb7, 0xca, 0x51,
					    0x67, 0x45, 0xbf, 0xea, 0xfc,
					    0x49, 0x90, 0x4b, 0x49, 0x60,
					    0x89 };

	printf("test vectors. please look doc/fips-test-vectors.txt\n");

	printf("testing 128bit key...\n");
	encrypt_test(key, 128, pt, 16, ct, ct_result_128);
	decrypt_test(key, 128, pt, 16, ct); /* crypted text is 16 + pad */
	printf("testing 192bit key...\n");
	encrypt_test(key, 192, pt, 16, ct, ct_result_192);
	decrypt_test(key, 192, pt, 16, ct);
	printf("testing 256bit key...\n");
	encrypt_test(key, 256, pt, 16, ct, ct_result_256);
	decrypt_test(key, 256, pt, 16, ct);
	printf("done.\n");
	return 0;
}

